import java.util.List;

import javax.persistence.EntityNotFoundException;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;

import pkg.Customer;
import pkg.Order;

/** */
public class Main {
    
    private EntityManagerFactory emf;

    public static void main (String[] args) {
        Main main = new Main();
        main.testWithSameEM();
        main.testWithSameEMAndClear();
        main.testWithNewEM();
    }
    
    public Main () {
        // create EMF
        emf = Persistence.createEntityManagerFactory("j2seEnvironment");
    }

    void testWithSameEM() {
        EntityManager em = emf.createEntityManager();
        try {
            System.out.println("\n### Run test with the same EntityManager");
            createData(em);
            delete(em);
            testDB(em);
            access(em);
        } finally {
            if ((em != null) && em.isOpen()) {
                em.close();
            }
            cleanupDB();
        }
    }

    void testWithSameEMAndClear() {
        EntityManager em = emf.createEntityManager();
        try {
            System.out.println("\n### Run test with the same EntityManager and execute clear after delete.");
            createData(em);
            delete(em);
            testDB(em);
            em.clear();
            access(em);
        } finally {
            if ((em != null) && em.isOpen()) {
                em.close();
            }
            cleanupDB();
        }
    }

    void testWithNewEM() {
        EntityManager em = null;
        try {
            System.out.println("\n### Run test with one EntityManager per create/delete/access operation.");
            em = emf.createEntityManager();
            createData(em);
            em.close();
            em = emf.createEntityManager();
            delete(em);
            em.close();
            em = emf.createEntityManager();
            testDB(em);
            em.close();
            em = emf.createEntityManager();
            access(em);
        } finally {
            if ((em != null) && em.isOpen()) {
                em.close();
            }
            cleanupDB();
        }
    }

    void createData(EntityManager em) {
        EntityTransaction t = em.getTransaction();  
        try {
            t.begin();
            // create Customer
            Customer michael = new Customer();
            michael.setId(1);
            michael.setName("Michael");

            // create Orders
            for (int i = 1; i <= 3; i++) {
                Order o = new Order();
                o.setId(i);
                o.setTotalPrice(i*100);
                michael.addOrder(o);
                em.persist(o);
            }
            em.persist(michael);
            t.commit();
        } finally {
            if ((t != null) && t.isActive()) {
                t.rollback();
            }
        }
    }

    void delete(EntityManager em) {
        EntityTransaction t = em.getTransaction();  
        try {
            t.begin();
            int deleted = em.createQuery("DELETE FROM Order o WHERE o.id = 1").executeUpdate();
            System.out.println("### No of deleted orders " + deleted);
            t.commit();
        } finally {
            if ((t != null) && t.isActive()) {
                t.rollback();
            }
        }
    }
    
    void access(EntityManager em) {
        EntityTransaction t = em.getTransaction();  
        try {
            t.begin();
            Customer c = em.find(Customer.class, new Integer(1));
            System.out.println("### Found Customer " + c);
            System.out.println("    with orders ");
            for (Order o : c.getOrders()) {
                System.out.println("        " + o + " " + em.contains(o));
                if (o.getId() == 1) { // the removed instance
                    try {
                        em.refresh(o);
                        System.out.println("### missing EntityNotFoundException");
                    } catch (EntityNotFoundException ex) {
                        // expected exception
                    }
                }
            }
            t.commit();
        } finally {
            if ((t != null) && t.isActive()) {
                t.rollback();
            }
        }
    }
    
    void testDB(EntityManager em) {
        System.out.println("### No of order rows in the database: " + 
                           em.createNativeQuery("SELECT COUNT(*) FROM ORDER_TABLE").getSingleResult());
    }

    void cleanupDB() {
        EntityManager em = emf.createEntityManager();
        EntityTransaction t = em.getTransaction(); 
        try {
            t.begin();
            em.createQuery("DELETE FROM Order").executeUpdate();
            em.createQuery("DELETE FROM Customer").executeUpdate();
            t.commit();
        } finally {
            if ((t != null) && t.isActive()) {
                t.rollback();
            }
            if((em != null) && em.isOpen()) {
                em.close();
            }
        }
    }
}
